package cz.muni.fi.pa165.controller;

import cz.muni.fi.pa165.event.ProposalAcceptedEvent;
import cz.muni.fi.pa165.model.ProposalDTO;
import cz.muni.fi.pa165.service.ProposalService;
import jakarta.persistence.EntityNotFoundException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.*;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.oauth2.server.resource.authentication.BearerTokenAuthentication;
import org.springframework.security.oauth2.server.resource.introspection.OAuth2IntrospectionAuthenticatedPrincipal;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.HttpClientErrorException;
import org.springframework.web.client.RestTemplate;

import java.util.List;

@RestController
@Slf4j
public class ProposalControllerImpl implements ProposalController {
    private final ProposalService proposalService;
    private final RestTemplate restTemplate;

    private final KafkaTemplate<String, ProposalAcceptedEvent> kafkaTemplate;

    @Autowired
    public ProposalControllerImpl(ProposalService proposalService, RestTemplate restTemplate, KafkaTemplate<String, ProposalAcceptedEvent> kafkaTemplate) {
        this.proposalService = proposalService;
        this.restTemplate = restTemplate;
        this.kafkaTemplate = kafkaTemplate;
    }

    @Override
    public ResponseEntity<ProposalDTO> createProposal(ProposalDTO proposalDTO) {
        var userId = proposalDTO.getUserId();
        var bandId = proposalDTO.getBandId();

        BearerTokenAuthentication authentication = (BearerTokenAuthentication)SecurityContextHolder.getContext().getAuthentication();
        String accessToken = authentication.getToken().getTokenValue();
        HttpHeaders headers = new HttpHeaders();
        headers.setBearerAuth(accessToken);
        HttpEntity<String> entity = new HttpEntity<>(headers);

        // Check if the user exists
        try {
            ResponseEntity<Object> userResponseEntity = restTemplate.exchange("http://pa165-user-service:9091/api/v1/users/{userId}", HttpMethod.GET, entity, Object.class, userId);
        } catch (HttpClientErrorException e) {
            if (e.getStatusCode().equals(HttpStatus.NOT_FOUND)) {
                throw new EntityNotFoundException("User with ID " + userId + " was not found");
            }
        }
        // Check if the band exists
        try {
            ResponseEntity<Object> bandResponseEntity = restTemplate.exchange("http://pa165-band-management-service:9092/api/v1/bands/{bandId}", HttpMethod.GET, entity, Object.class, bandId);
        } catch (HttpClientErrorException e) {
            if (e.getStatusCode().equals(HttpStatus.NOT_FOUND)) {
                throw new EntityNotFoundException("Band with ID " + bandId + " was not found");
            }
        }

        return ResponseEntity.status(HttpStatus.CREATED).body(proposalService.createProposal(proposalDTO));
    }

    @Override
    public ResponseEntity<ProposalDTO> getProposalById(long id) {
        return ResponseEntity.ok(proposalService.getProposalById(id));
    }
    @Override
    public ResponseEntity<Void> acceptProposal(long proposalId) {
        ProposalDTO found = proposalService.getProposalById(proposalId);
        proposalService.resolveProposal(proposalId);

        ProposalAcceptedEvent proposalAcceptedEvent = new ProposalAcceptedEvent(found.getUserId(), found.getBandId());
        kafkaTemplate.send("proposalAcceptedTopic", proposalAcceptedEvent);
        log.info("Sent proposal accepted event: {}", proposalAcceptedEvent);

        return ResponseEntity.noContent().build();
    }

    @Override
    public ResponseEntity<Void> rejectProposal(long proposalId) {
        proposalService.resolveProposal(proposalId);
        return ResponseEntity.noContent().build();
    }

    @Override
    public ResponseEntity<List<ProposalDTO>> getProposalsByUserId(String userId) {
        return ResponseEntity.ok(proposalService.getProposalsByUserId(Long.parseLong(userId)));
    }
}
